/* source: xio-ext2.h */
/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
/* Published under the GNU General Public License V.2, see file COPYING */

#ifndef __xio_ext2_h_included
#define __xio_ext2_h_included 1

extern const struct optdesc opt_ext2_secrm;
extern const struct optdesc opt_ext2_unrm;
extern const struct optdesc opt_ext2_compr;
extern const struct optdesc opt_ext2_sync;
extern const struct optdesc opt_ext2_immutable;
extern const struct optdesc opt_ext2_append;
extern const struct optdesc opt_ext2_nodump;
extern const struct optdesc opt_ext2_noatime;
extern const struct optdesc opt_ext2_journal_data;
extern const struct optdesc opt_ext2_notail;
extern const struct optdesc opt_ext2_dirsync;
extern const struct optdesc opt_ext2_topdir;

#endif /* !defined(__xio_ext2_h_included) */
