/* source: xiosysincludes.h */
/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
/* Published under the GNU General Public License V.2, see file COPYING */

#ifndef __xiosysincludes_h_included
#define __xiosysincludes_h_included 1

#include "config.h"
#include "xioconfig.h"	/* what features are enabled */

#include "sysincludes.h"

#endif /* !defined(__xiosysincludes_h_included) */
