/* source: procan.h */
/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
/* Published under the GNU General Public License V.2, see file COPYING */

#ifndef __procan_h_included
#define __procan_h_included 1

extern int procan(FILE *outfile);
extern int procan_cdefs(FILE *outfile);

#endif /* !defined(__procan_h_included) */
